/* 
Copyright  2003  
   Dauger Research, Inc. 
   http://daugerresearch.com/
*/

#ifndef __poochcljobinput

#define __poochcljobinput

#define MaxFiles	256
#define MaxNodes	256
#define MaxExtraArgs	256

#define ForegroundFlag		1L

typedef struct JobOptionsStruct {
	char *jobFileData; 
	long numFileArgs;
	char *fileArgs[MaxFiles]; 
	long numNodeArgs;
	char *nodeArgs[MaxNodes]; 
	long numExtraArgs;
	char *extraArgs[MaxExtraArgs];
	int numProcessorOption; 
	int jobFileArg; 
	char *subDirArg; 
	long numNodesRequested; 
	long flags; 
	short proceed, iAmLaunching; 
	short jobFileFlag, noSelfOption; 
	short skipBusyCheck, launchFailureQueuesJob;
	short subdirectoryOption, waitForUserToLaunch;
	short jobType, testMode, saveScript, tasksPerComputer; 
	short autoRetrieve; 
	unsigned short tcpPort; 
	char *dataStorage; 
	char *pathContext; 
/*	int argc;
	char **argv; */
	int unused; 
	} JobOptionsStruct, *JobOptionsPtr, **JobOptionsHandle; 

/* Reads a null-terminated long string in data and parses its arguments into a series of 
null-terminated strings in dataStorage[0] and pointed to by argvOut[0].
Note: dataStorage[0] and argvOut[0] should be free()-d later after use. */
void parseRawDataIntoArguments(char *data, char **dataStorage, int *argcOut, char **argvOut[]); 
void parseRawDataIntoArguments(char *data, char **dataStorage, int *argcOut, char **argvOut[])
	{int argc=0; 
	char **argv=NULL; 
if (data&&dataStorage) 
	{
	long length=strlen(data); 
	
	if (length>0) {
		if (length>0) {
			dataStorage[0]=(char*)malloc(length); 
			
			if (dataStorage[0]) {
				long i; 
				argc=0; 
				char *inp=data; 
				char *tp=dataStorage[0]; 
				
				tp[0]=0; 
				
				/* parse it */
				while (sscanf(inp, "%s", tp)>0) {
					argc++; 
					inp=strstr(inp, tp); 
					inp+=strlen(tp); 
					tp+=strlen(tp)+1; 
					}
					
				printf("parsing data: has %ld arguments\n", argc); 
				
				argc++; 
				argv=(char **)malloc(sizeof(char *)*(argc+1)); 
				argv[0]=tp=dataStorage[0]; 
				for(i=1; i<argc; i++) {
					argv[i]=tp; 
					tp+=strlen(tp)+1; 
					}
				argv[argc]=NULL; 
				
				}
				
			}
									
		}
	}
	if (argcOut) argcOut[0]=argc; 
	if (argvOut) argvOut[0]=argv; 
	}
	
void generateJobFileSample(); 
void generateJobFileSample()
	{
	printf("\n"); 
	printf("executable\nfile1\nfile2\n...\n-n\naddr1\naddr2\n...\n-options\n"); 
	printf("\n\tor, using the \"best\" criteria, ...\n\n");
	printf("executable\nfile1\nfile2\n...\n-n \nnodeCount\n-options\n"); 
	printf("\n"); 
	}


int parseJobArguments(int argc, char *argv[], JobOptionsPtr optionsP); 
int parseJobArguments(int argc, char *argv[], JobOptionsPtr optionsP)
	{int out=0; 
	
	if (!optionsP) return 0; 
	if (!argv) return 0; 
	
	if (optionsP) {
		
#ifdef ForegroundMode
		optionsP->flags|=ForegroundFlag; 
#endif
		
	if (argc<=1) optionsP->proceed=0; 
	else {
		int i; 
		short readingNodes=0; 
		optionsP->proceed=out=1; 
#ifdef DefaultJobType
		optionsP->jobType=DefaultJobType; 
#else
		optionsP->jobType=0; //MacMPI
#endif
		
		for(i=1; i<argc; i++) {/* item by item */
			short unrecognizedOption=0, notADashOption=0; 
			
			if ('-'==argv[i][0]) {/* an option */
				char *optS=&argv[i][1]; 
				
				if (strlen(optS)>1) {
#define	ifOptIs(str)	if(!strcmp(optS,str))
					
					ifOptIs("np") {//number of processors
						i++; //next option
						if (i<argc) {
							if (optionsP->numNodeArgs<MaxNodes) {
								optionsP->nodeArgs[optionsP->numNodeArgs++]=argv[i]; 
								if (!optionsP->numProcessorOption) {
									sscanf(argv[i], "%d", &optionsP->numProcessorOption); 
									}
								}
							}
						else 
							printf("no specification for -np\n"); 
						}
					else ifOptIs("nodes") {//number of nodes
						i++; //next option
						if (i<argc) {
							if (optionsP->numNodeArgs<MaxNodes) 
								optionsP->nodeArgs[optionsP->numNodeArgs++]=argv[i]; 
							if (!optionsP->numNodesRequested) 
								sscanf(argv[i], "%ld", &optionsP->numNodesRequested); 
							}
						else 
							printf("no specification for -nodes\n"); 
						}
					else ifOptIs("nolocal") {//not me
						optionsP->noSelfOption=1; 
						}
					else ifOptIs("machinefile") {//machinefile format
						
						}
					else if(!strcmp(optS,"allcpus")||!strcmp(optS,"all-cpus")) {
						//all cpus on all nodes
						if (1) {
							char *newStr=(char*)malloc(16); 
							if (newStr) {
								sprintf(newStr, "65536"); 
								if (!optionsP->numNodeArgs) optionsP->numNodeArgs++; 
								optionsP->nodeArgs[0]=newStr; 
								argv[i]=newStr; 
								}
							}
						else printf("no specification for %s\n", argv[i-1]); 
						}
					else ifOptIs("tcp") {//TCP port number specification
						i++; //next option
						if (i<argc) {
							if (!optionsP->tcpPort) {
								int tcpPort=0; 
								sscanf(argv[i], "%d", &tcpPort); 
								optionsP->tcpPort=tcpPort; /* 0 means default behavior */
								}
							}
						else printf("no specification for %s\n", argv[i-1]); 
						}
					else ifOptIs("help") {//print help
						optionsP->proceed=0; 
						out=0; 
						}
					else ifOptIs("-------") {//generated by poochqueue: the node list
						readingNodes=1; 
						}
					else {
						unrecognizedOption=1; 
						printf("Unrecognized option: %s\n", optS); 
						}
					
#undef ifOptIs
					}
				else //single letter or null 
					switch (optS[0]) {
						case '-': //--
						case 0: //-
						case 'h': //-help
						case '?': //-?
							optionsP->proceed=0; 
							out=0; 
							break; 
						case 's': 
								optionsP->saveScript=1; 
							if (!(ForegroundFlag&optionsP->flags)) {
								optionsP->proceed=0; 
								out=0; 
								}
							break; 
						case 'm': optionsP->noSelfOption=1; break; 
						case 'f': readingNodes=0; break; 
						case 'n': readingNodes=1; break; 
						case 'd': optionsP->subdirectoryOption=1; break; 
						case 'r': optionsP->autoRetrieve=-1; break; 
						case 'p': optionsP->tasksPerComputer=-1; break; 
						case 't': 
								optionsP->testMode=1; 
							if (!(ForegroundFlag&optionsP->flags)) {
								optionsP->proceed=0; 
								out=0; 
								}
							break; 
						case 'T': optionsP->jobType=-1; break; 
						case 'K': optionsP->skipBusyCheck=1; break; 
						case 'Q': optionsP->launchFailureQueuesJob=1; break; 
						case 'W': 
							optionsP->waitForUserToLaunch=1; 
							if (!(ForegroundFlag&optionsP->flags)) {
								optionsP->proceed=0; 
								out=0; 
								}
							break; 
						case 'j': /* only if first argument */
							if (!(ForegroundFlag&optionsP->flags)) {
								optionsP->proceed=0; 
								out=0; 
								}
							else {
							if (1==i) if (!optionsP->jobFileData) {
								
								if (argc<3) {
									optionsP->proceed=0; 
									generateJobFileSample(); 
									}
								else {
									FILE *jfp=fopen(argv[2], "r"); 
									long length=0; 
									
									if (jfp) {
										fseek(jfp, 0, SEEK_END); 
										length=ftell(jfp); 
										if (length>0) {
											optionsP->jobFileData=(char*)malloc(length); 
											
											if (optionsP->jobFileData) {
												long argCount=0; 
												char *tp=optionsP->jobFileData; 
												
												tp[0]=0; 
												
												rewind(jfp); /* read it all in */
												/* and parse it */
												while (EOF!=fscanf(jfp, "%s", tp)) {
													argCount++; 
													tp+=strlen(tp)+1; 
													}
												
												printf("parsing job file: %s has %ld arguments\n", argv[2], argCount); 
												
												argc=argCount+1; 
												argv=(char **)malloc(sizeof(char *)*(argc+1)); 
												argv[0]=tp=optionsP->jobFileData; 
												for(i=1; i<argc; i++) {
													argv[i]=tp; 
													tp+=strlen(tp)+1; 
													}
												argv[argc]=NULL; 
												
												optionsP->jobFileFlag=1; 
												i=0; /*start over*/
												}
											
											}
										
										fclose(jfp); 
										}
									}
								}
								
								}
							break; 
						default: unrecognizedOption=1; 
							printf("Unrecognized option: %s\n", optS); 
							break; 
					}
				}
			else 
				notADashOption=1; 
			
			if (unrecognizedOption||notADashOption) {
				
				if (optionsP->subdirectoryOption&&!optionsP->subDirArg) {
					optionsP->subDirArg=argv[i]; 
					}
				else 
				if (-1==optionsP->jobType) {
					if (!strcmp("lammpi", argv[i])) 
						optionsP->jobType=4; 
					else if (!strcmp("LAM", argv[i])) 
						optionsP->jobType=4; 
					else if (!strcmp("lam", argv[i])) 
						optionsP->jobType=4; 
					else if (!strcmp("mpichgm", argv[i])) 
						optionsP->jobType=3; 
					else if (!strcmp("mpich", argv[i])) 
						optionsP->jobType=1; 
					else if (!strcmp("MPIPro", argv[i])) 
						optionsP->jobType=2; 
					else if (!strcmp("MacMPI", argv[i])) 
						optionsP->jobType=0; 
					else 
						optionsP->jobType=0; 
					}
				else 
				if (-1==optionsP->autoRetrieve) {
					if (!strcmp("all", argv[i])) 
						optionsP->autoRetrieve=3; 
					else if (!strcmp("off", argv[i])) 
						optionsP->autoRetrieve=1; 
					else if (!strcmp("0", argv[i])) 
						optionsP->autoRetrieve=0; 
					else 
						optionsP->autoRetrieve=0; 
					}
				else 
				if (-1==optionsP->tasksPerComputer) {
					int tasks; 
					sscanf(argv[i], "%d", &tasks); 
					if (tasks>=0) 
						optionsP->tasksPerComputer=tasks; 
					else 
						optionsP->tasksPerComputer=0; 
					}
				else {
				
				if (optionsP->numFileArgs>0) {//must already have at least one file
					if (optionsP->numExtraArgs<MaxExtraArgs) 
						optionsP->extraArgs[optionsP->numExtraArgs++]=argv[i]; 
						//use whatever comes: extra files or unrecognized arguments
					}
				
				if (notADashOption) 
				if (!readingNodes) {
					if (optionsP->numFileArgs<MaxFiles) 
						optionsP->fileArgs[optionsP->numFileArgs++]=argv[i]; 
					}
				else {
					if (optionsP->numNodeArgs<MaxNodes) 
						optionsP->nodeArgs[optionsP->numNodeArgs++]=argv[i]; 
					if (!optionsP->numNodesRequested) 
						sscanf(argv[i], "%ld", &optionsP->numNodesRequested); 
					}
				
				}
				
				}
			
			}
		}
		
		}
	
	return out; 	
	}

int parseJobArgumentsFromData(char *jobData, JobOptionsPtr optionsP); 
int parseJobArgumentsFromData(char *jobData, JobOptionsPtr optionsP)
	{int out=0; 
	
	if (!jobData) return 0; 
	if (!optionsP) return 0; 
	
	if (optionsP) {
		int argc=0;
		char **argv=NULL; 
		
		{long *tp=(long*)optionsP, i=sizeof(JobOptionsStruct)>>2; 
		while (i--) tp[i]=0; /* clear it */
		}
	
		optionsP->dataStorage=NULL; 
		parseRawDataIntoArguments(jobData, &optionsP->dataStorage, &argc, &argv); 
	
		out=parseJobArguments(argc, argv, optionsP); 
		
		if (argv) free(argv); 
		}
	
	return out; 	
	}

int parseJobArgumentsFromMainArgs(int argc, char *argv[], JobOptionsPtr optionsP); 
int parseJobArgumentsFromMainArgs(int argc, char *argv[], JobOptionsPtr optionsP)
	{int out=0; 
	
	if (!argv) return 0; 
	if (!optionsP) return 0; 
	
	if (optionsP) {
		
		{long *tp=(long*)optionsP, i=sizeof(JobOptionsStruct)>>2; 
		while (i--) tp[i]=0; /* clear it */
		}
	
		optionsP->dataStorage=NULL; 
	
		out=parseJobArguments(argc, argv, optionsP); 
		
		}
	
	return out; 	
	}

#endif